class TeachersController < ApplicationController

  before_action :set_teacher, only: [:show, :edit, :edit_roles,:update_roles, :update, :destroy]

  def index
    authorize!(:read,Teacher)
    add_breadcrumb "Lehrkräfte", :teachers_path
    @grid = TeacherGrid.new(params[:teacher_grid]) do |scope|
      scope.accessible_by(current_ability).page(params[:page]).per(25)
    end
  end

  def show
    authorize!(:read,@teacher)
    add_breadcrumb "Lehrkräfte", :teachers_path
    add_breadcrumb "#{view_context.show_person(@teacher)}", teacher_path(@teacher)
  end

  def new
    authorize!(:create,Teacher)
    add_breadcrumb "Lehrkräfte", :teachers_path
    add_breadcrumb "neu", :new_teacher_path
    @teacher = Teacher.new(params.fetch(:teacher, {}).permit(teacher_institution_relations_attributes: [:institution_id]))
    @teacher.identity = User.new
    @teacher.teacher_institution_relations.build unless @teacher.teacher_institution_relations.any?
  end

  def edit
    add_breadcrumb "Lehrkräfte", :teachers_path
    add_breadcrumb "#{view_context.show_person(@teacher)}", teacher_path(@teacher)
    add_breadcrumb "bearbeiten", edit_teacher_path(@teacher)
    @teacher.build_identity if @teacher.identity.nil?
    authorize!(:update,@teacher)
  end

  def edit_roles
    if @teacher.identity.nil?
      flash[:danger] = 'Der Lehrer kann sich gar nicht einloggen, daher könne auch keine Rollen vergeben werden'
      redirect_to :back and return
    end
    add_breadcrumb "Lehrkräfte", :teachers_path
    add_breadcrumb "#{view_context.show_person(@teacher)}", teacher_path(@teacher)
    add_breadcrumb "Rollen bearbeiten", edit_teacher_path(@teacher)
    authorize!(:update_roles,@teacher)
  end

  def update_roles
    if @teacher.identity.nil?
      flash[:danger] = 'Der Lehrer kann sich gar nicht einloggen, daher könne auch keine Rollen vergeben werden'
      redirect_to :back and return
    end
    authorize!(:update_roles,@teacher)
    add_breadcrumb "Lehrkräfte", :teachers_path
    add_breadcrumb "#{view_context.show_person(@teacher)}", teacher_path(@teacher)
    add_breadcrumb "Rollen bearbeiten", edit_teacher_path(@teacher)
    respond_to do |format|
      if @teacher.identity.update(teacher_update_roles_params)
        format.html { redirect_to teachers_path, success: 'Gespeichert' }
        format.json { render :show, status: :ok, location: @teacher }
      else
        flash[:danger] =  'Nicht gespeichert'
        format.html { render :edit_roles }
        format.json { render json: @teacher.errors, status: :unprocessable_entity }
      end
    end
  end

  def create
    add_breadcrumb "Lehrkräfte", :teachers_path
    add_breadcrumb "neu", :new_teacher_path    
    @teacher = Teacher.new(teacher_params)
    authorize!(:create,@teacher)
    respond_to do |format|
      if @teacher.save
        format.html { redirect_to teachers_path, success: 'Gespeichert' }
        format.json { render :show, status: :created, location: @teacher }
      else
        @teacher.build_identity if @teacher.identity.nil?
        flash[:danger] = 'Bitte prüfen Sie die Eingaben'
        format.html { render :new }
        format.json { render json: @teacher.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    authorize!(:update,@teacher)
    add_breadcrumb "Lehrkräfte", :teachers_path
    add_breadcrumb "#{view_context.show_person(@teacher)}", teacher_path(@teacher)
    add_breadcrumb "bearbeiten", edit_teacher_path(@teacher)    
    respond_to do |format|
      if @teacher.update(teacher_params)
        format.html { redirect_to teachers_path, success: 'Gespeichert' }
        format.json { render :show, status: :ok, location: @teacher }
      else
        @teacher.build_identity if @teacher.identity.nil?
        flash[:danger] =  'Nicht gespeichert'
        format.html { render :edit }
        format.json { render json: @teacher.errors, status: :unprocessable_entity }
      end
    end
  end

  def destroy
    authorize!(:destroy,@teacher)
    @teacher.destroy
    respond_to do |format|
      format.html { redirect_to teachers_url, success: 'Gelöscht' }
      format.json { head :no_content }
    end
  end



  private
    # Use callbacks to share common setup or constraints between actions.
    def set_teacher
      @teacher = Teacher.find(params[:id])
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def teacher_params
      params.require(:teacher).permit(
        :first_name, :teacher_category_id, :last_name, :gender,
        :email, :phone, :mobile, :fax, :website, :can_log_in,
        identity_attributes: [:email, :password, :password_confirmation],
        teacher_institution_relations_attributes: [
          :id, :institution_id, :special_education_part_time, :normal_education_part_time, :spare_time,
          :teacher_institution_category_id, :portfolio_specialization_id, :_destroy
        ] )
    end

    def teacher_update_roles_params
      unless current_user.super_admin?
        return params.require(:user).permit(auth_role_ids:[])
      else
        return params.require(:user).permit(:super_admin, auth_role_ids:[])
      end
    end

end
