class CareerItem < ActiveRecord::Base
  #### die Laufbahn eines Klienten

  CHANGE_TYPE_CLIENT_GROUP  = 0
  CHANGE_TYPE_LEVEL         = 1
  CHANGE_TYPE_TEACHER       = 2
  CHANGE_TYPE_INSTITUTION   = 3
  CHANGE_TYPES = ["Klassenwechsel","Förderstufenwechsel","Unterstützung durch sonderpädagogische Fachkraft","Wechsel der Einrichtung"]

  #--- Concerns
  include Publishable

  #--- Associations
  belongs_to :client        # never NULL

  belongs_to :institution   # can be NULL
  belongs_to :client_group  # can be NULL
  belongs_to :teacher       # can be NULL

  #--- Scopes
  scope :client_group_changes,    -> { where(change_type: CHANGE_TYPE_CLIENT_GROUP) }
  scope :portfolio_level_changes, -> { where(change_type: CHANGE_TYPE_LEVEL) }
  scope :teacher_id_changes,      -> { where(change_type: CHANGE_TYPE_TEACHER) }
  scope :institution_changes,     -> { where(change_type: CHANGE_TYPE_INSTITUTION) }
  scope :default_order,           -> { order(start_at: :desc,created_at: :desc) }

  #--- Validations ( actually not necessary, no user submitted data  )
  validates :client_id, :change_type, presence: true # is dependent on client
  validates :change_type, inclusion: [CHANGE_TYPE_CLIENT_GROUP,CHANGE_TYPE_INSTITUTION,CHANGE_TYPE_LEVEL,CHANGE_TYPE_TEACHER]

  #--- Restricted on create
  validates :end_at,            absence: true, on: :create
  validates :start_at,          absence: true, on: :create
  validates :portfolio_level,   absence: true, on: :create
  validates :client_group_id,   absence: true, on: :create
  validates :client_group_name, absence: true, on: :create
  validates :teacher_id,        absence: true, on: :create
  validates :teacher_name,      absence: true, on: :create
  validates :institution_id,    absence: true, on: :create
  validates :institution_name,  absence: true, on: :create

  validate on: :update do
    if effective_date_changed?
      if end_at.nil? && effective_date > Date.today
        errors.add(:effective_date,'Dies ist der jüngste Eintrag seiner Art. Das Datum des Inkrafttretens kann hier nur in der Vergangenheit gesetzt werden.')
      end
      if end_at.present? && effective_date > end_at
        errors.add(:effective_date,'Das Datum kann nicht später als der nächste Eintrag dieser Art gesetzt werden.')
      end
      _previous = CareerItem.where(client_id: client_id, change_type: change_type).where('end_at <= ?',start_at).order(end_at: :desc,created_at: :desc).first
      if _previous.present? && effective_date < _previous.start_at
        errors.add(:effective_date,'Das Datum kann nicht vor dem letzten Eintrag dieser Art gesetzt werden.')
      end
    end
  end

  #--- Restricted on update
  attr_readonly :end_at, :start_at, :portfolio_level, :client_group_id, :client_group_name, :teacher_id, :teacher_name, :institution_id, :institution_name, :change_type




  #--- Callback definitions
  before_create :set_defaults_for_new
  before_create :end_earlier_record # this ensures there is a gapless chronological history
  after_create :discard_older_changes_on_same_day # this prevents multiple changes in short time to be reflected in history





  # DESTRUCTION: if career_items are deleted, the full career of a client must be deleted. There cannot be single entries deleted for a client.
  def self.portfolio_level_at_date(client,date = Date.today())
    career_item = where(client_id: client.id, change_type: CHANGE_TYPE_LEVEL).where('start_at <= ? AND (end_at > ? OR end_at IS NULL )',date,date).take
    return 0 if career_item.blank?
    return career_item.portfolio_level.to_i
  end

  def self.max_portfolio_level_after_date(client,date = Date.today())
    max = where(client_id: client.id, change_type: CHANGE_TYPE_LEVEL).where("start_at >= ?",date).maximum(:portfolio_level)
    return 0 unless max.present?
    return max
  end


  private

    # before_create
    def set_defaults_for_new
      # each record reflects the current state of the user
      self.start_at = Date.today
      self.end_at = nil
      self.effective_date = start_at

      # on create, the record is set to the current state of the client record
      self.teacher_id = client.teacher_id
      self.client_group_id = client.client_group_id
      self.portfolio_level = client.portfolio_level
      self.institution_id = client.institution_id

      # the name fields are filled
      unless institution_id.blank?
        self.institution_name = institution.name
      end

      unless teacher_id.blank?
        self.teacher_name = "#{teacher.first_name} #{teacher.last_name}"
      end

      unless client_group_id.blank?
        self.client_group_name = client_group.name
      end
    end

    # before_create
    def end_earlier_record
      self.class.where(client_id: client_id, change_type: change_type, end_at: nil ).update_all(end_at: start_at)
    end

    # after_create
    def discard_older_changes_on_same_day
      self.class.where(client_id: client_id, change_type: change_type, start_at: start_at, end_at: start_at ).destroy_all
    end




end
