class Notification < ActiveRecord::Base

  IDENTIFIER_MANAGEMENT_CONFERENCE                    = 'ManagementConference'
  IDENTIFIER_PARENT_TEACHER_CONFERENCE                = 'ParentTeacherConference'
  IDENTIFIER_CLIENT_GROUP_MISSING                     = 'SemesterEndsClientGroupMissing'
  IDENTIFIER_CLIENT_CONFIDENTIALITY_DOCUMENT_MISSING  = 'ClientConfidentialityDocumentMissing'
  IDENTIFIER_CLIENT_DUPLICATE_CREATE_REQUEST          = 'ClientDuplicateCreateRequest'
  IDENTIFIER_CLIENT_DUPLICATE_DUPLICATE_REQUEST       = 'ClientDuplicateDuplicateRequest'
  IDENTIFIER_CLIENT_DUPLICATE_REQUEST_CLIENT_CREATED  = 'ClientDuplicateRequestCreated'
  IDENTIFIER_DIAGNOSTIC_RESULT_INCOMPLETE             = 'DiagnosticResultIncomplete'
  PREFIX_IDENTIFIER_PORTFOLIO_RULE                    = 'PortfolioRule'
  IDENTIFIER_CLIENT_VIOLATED_COURSE_RULE              = 'ViolatedCourseRule'
  IDENTIFIER_BORROW_PORTFOLIO_DIAGNOSTIC              = 'BorrowPortfolioDiagnostic'
  IDENTIFIER_BORROW_PORTFOLIO_ADVANCEMENT_PLAN        = 'BorrowPortfolioAdvancementPlan'


  #--- Behavior
  #acts_as_paranoid # INFO: dient zu Controlling-Zwecken für Admins

  #--- Associations
  # this is the main subject of notifications, it is used for resolving and filtering of notifications and should always be set
  belongs_to :notificationable, polymorphic: true

  # this can be set, if necessary for resolving.
  belongs_to :secondary_notificationable, polymorphic: true

  belongs_to :client
  belongs_to :institution

  #--- Scopes
  default_scope { order(prio: :desc, due_at: :asc) }
  scope :only_type, ->(type) {
    where("notificationable_type in ?", type.map(&:camelize) ) if type.is_a? Array
    where(notificationable_type: type.camelize) unless type.is_a? Array
  }
  scope :only_showable, -> { where('start_at <= ?',DateTime.now) }

  def is_manual_confirmable?
    # These require certain actions for resolve:
    return false if identifier == IDENTIFIER_MANAGEMENT_CONFERENCE
    return false if identifier == IDENTIFIER_PARENT_TEACHER_CONFERENCE
    return false if identifier == IDENTIFIER_CLIENT_GROUP_MISSING
    return false if identifier == IDENTIFIER_CLIENT_DUPLICATE_DUPLICATE_REQUEST
    return false if identifier == IDENTIFIER_CLIENT_VIOLATED_COURSE_RULE
    return false if identifier.start_with? PREFIX_IDENTIFIER_PORTFOLIO_RULE

    # These can be just clicked away:
    return true  if identifier == IDENTIFIER_CLIENT_DUPLICATE_CREATE_REQUEST
    return true  if identifier == IDENTIFIER_CLIENT_DUPLICATE_REQUEST_CLIENT_CREATED
    return true  if identifier == IDENTIFIER_BORROW_PORTFOLIO_DIAGNOSTIC
    return true  if identifier == IDENTIFIER_BORROW_PORTFOLIO_ADVANCEMENT_PLAN
  end

  # zugehörigkeit, für anzeige in views
  def label
    case notificationable
      when ::Portfolio::Portfolio
        return notificationable.portfolio_subject.name
      when ::Portfolio::Item
        return notificationable.portfolio_subject.name
      when ::Client
        return "#{notificationable.first_name} #{notificationable.last_name}"
      when ::ClientGroup
        return "Klasse #{notificationable.name}"
      when ::ClientSupporterRelation
        return "#{notificationable.client.first_name} #{notificationable.client.last_name}"
      when ::ClientDuplicateRequest
        return "#{notificationable.first_name} #{notificationable.last_name}"
    end
    return"#{notificationable.class == ::Portfolio::Item}"
  end

  def confirm_at_path
    routes = Rails.application.routes.url_helpers
    return routes.select_portfolio_item_date_clients_path(client_id: client_id) if identifier == IDENTIFIER_MANAGEMENT_CONFERENCE
    return routes.select_portfolio_item_date_clients_path(client_id: client_id) if identifier == IDENTIFIER_PARENT_TEACHER_CONFERENCE
    return routes.edit_next_semester_client_groups_clients_path if identifier == IDENTIFIER_CLIENT_GROUP_MISSING
    return :back if identifier == IDENTIFIER_CLIENT_DUPLICATE_CREATE_REQUEST
    return routes.dashboard_client_path(id: client_id) if identifier == IDENTIFIER_CLIENT_DUPLICATE_REQUEST_CLIENT_CREATED
    return routes.dashboard_client_path(id: client_id) if identifier == IDENTIFIER_CLIENT_VIOLATED_COURSE_RULE
    return routes.client_duplicate_request_path(notificationable) if identifier == IDENTIFIER_CLIENT_DUPLICATE_DUPLICATE_REQUEST
    if identifier.start_with? PREFIX_IDENTIFIER_PORTFOLIO_RULE
      portfolio = ::Portfolio::Portfolio.where(id: notificationable_id).take
      return portfolio.portfolio_subject.resolve_notifications_path
    end
    return routes.edit_client_supporter_relation_path(notificationable) if identifier == IDENTIFIER_CLIENT_CONFIDENTIALITY_DOCUMENT_MISSING
    return routes.edit_portfolio_item_path(notificationable) if identifier == IDENTIFIER_DIAGNOSTIC_RESULT_INCOMPLETE
    return routes.advancement_plan_path(catalog_id: secondary_notificationable.catalog_id) if identifier == IDENTIFIER_BORROW_PORTFOLIO_ADVANCEMENT_PLAN
    return routes.diagnostic_path(catalog_id: secondary_notificationable.catalog_id) if identifier == IDENTIFIER_BORROW_PORTFOLIO_DIAGNOSTIC

    return routes.root_path # unless any other matches
  end



  class << self
    def notifications_for_teacher(teacher_id)
      teacher = ::Teacher.where(id: teacher_id).take
      return ::Notification.none unless teacher
      return teacher.system_notifications
    end
    def notifications_for_client(client_id)
      client = ::Client.where(id: client_id).take
      return ::Notification.none unless client
      return client.system_notifications
    end
    def notifications_for_client_group(client_group)

      ids = client_group.notifications.ids

      client_group.clients.each do |client|
        ids << notifications_for_client(client.id).ids
      end

      ids << notifications_for_teacher(client_group.teacher_id).ids

      ids.flatten!
      where("id IN (?)", ids)
    end
  end


  #--- Attributes
  enum prio: { "gering": 0, "mittel": 1, "hoch": 2 }



end
