#!/bin/bash
#-----------------------------------------------------------------------------------------------------------------------
#
# Release Vschoolboard Application Server
#
# USAGE: release.sh --release-version=<version>
#-----------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------
# Check necessary commands
#-----------------------------------------------------------------------------------------------------------------------
#command -v tar >/dev/null 2>&1 || { echo "I require tar but it's not installed. Aborting." >&2; exit 1; }
command -v git >/dev/null 2>&1 || { echo "I require git but it's not installed. Aborting." >&2; exit 1; }
command -v gzip >/dev/null 2>&1 || { echo "I require gzip but it's not installed. Aborting." >&2; exit 1; }
command -v scp >/dev/null 2>&1 || { echo "I require scp but it's not installed. Aborting." >&2; exit 1; }

#-----------------------------------------------------------------------------------------------------------------------
# Script variables
#-----------------------------------------------------------------------------------------------------------------------
APP_INSTANZ="Vschoolboard Application Server"             # Application name
SCRIPT=$(readlink -f "$0")                                # scriptname
SCRIPTPATH=$(dirname "$SCRIPT")                           # scriptfolder
APP_RELEASE_VERSION=""                                    # the version to be released
APP_RELEASE_TAG=""                                     # the git branch that contains the release
INTERACTIVE=1
PUBLISH=0
#-----------------------------------------------------------------------------------------------------------------------
# Script parameters
#-----------------------------------------------------------------------------------------------------------------------
while [ $# -gt 0 ]; do
  case "$1" in
    --release-version=*)
      APP_RELEASE_VERSION="${1#*=}"
      ;;
    --release-branch=*)
      APP_RELEASE_TAG="${1#*=}"
      ;;
    --publish*)
      PUBLISH=1
      ;;
    --publish-to=*)
      RELEASE_LOCATION="${1#*=}"
      ;;
    --non-interactive*)
      INTERACTIVE=0
      ;;
    *)
      cat <<EOF
************************************************************************************************************************
USAGE: release.sh <options>

   Mandatory Options

   --release-version=<minor>.<major>.?<bugfix>
     release version. If is no bugfix, the part is omitted and not 0

   Other Options:

   --release-branch=<branchname>
     Defaults to v<minor>.<major>.<bugfix>
   --non-interactive
     run without user interaction
   --publish
     do publish ( default 0)
   --publish-to=<user>@<host>
     scp to publish destination
************************************************************************************************************************
EOF
      exit 1
  esac
  shift
done
if [[ -z ${RELEASE_LOCATION} ]]; then
  RELEASE_LOCATION="admin@82.165.153.43:/var/www/releases/" # where the release is pushed at the end. (ATTENTION: public key for admin!)
fi
if [[ -z ${APP_RELEASE_VERSION} ]]; then
  echo "Please set --release-version=<minor>.<major>.<bugfix>" 1>&2
  exit 1
fi
if [[ -z ${APP_RELEASE_TAG} ]]; then
  APP_RELEASE_TAG="v$APP_RELEASE_VERSION"
  echo "Using release tag $APP_RELEASE_TAG"
fi
if [[ -d .git/ ]]; then
  GIT_DIR=$(pwd)
else
  echo "$(pwd) seems to be not a git repository. Aborting..."
  exit 1
fi



#-----------------------------------------------------------------------------------------------------------------------
# check wether is bugfix or full upgrade
#-----------------------------------------------------------------------------------------------------------------------
RELEASE_WD="$SCRIPTPATH/releases"
echo "Preparing release in $RELEASE_WD"

if [[ ${INTERACTIVE} == 1 ]]; then
  echo -n "Proceed? (y/n)"
  read answer
  if ! echo "$answer" | grep -iq "^y" ; then
    exit 0
  fi
fi
mkdir -p ${RELEASE_WD}
cd ${RELEASE_WD}
if [[ -f "vsb-$APP_RELEASE_VERSION.tar.gz" ]]; then
  if [[ ${INTERACTIVE} == 1 ]]; then
    echo -n "vsb-$APP_RELEASE_VERSION.tar.gz exists. Proceed? (y/n)"
    read answer
    if ! echo "$answer" | grep -iq "^y" ; then
      exit 0
    fi
  fi
fi
if [[ $(cut -d'.' -f3 <<<"$APP_RELEASE_VERSION") > 0 ]]; then

  PREVIOUS_MINOR_RELEASE="$(cut -d'.' -f1 <<<"$APP_RELEASE_VERSION").$(cut -d'.' -f2 <<<"$APP_RELEASE_VERSION")"
  PREVIOUS_MINOR_RELEASE_TAG="v$PREVIOUS_MINOR_RELEASE"
  FOUND_PREVIOUS_MINOR_RELEASE=$(git -C ${GIT_DIR} tag -l "$PREVIOUS_MINOR_RELEASE_TAG")
  if [[ -z $FOUND_PREVIOUS_MINOR_RELEASE ]]; then
    PREVIOUS_MINOR_RELEASE_TAG="v$PREVIOUS_MINOR_RELEASE.0"
    FOUND_PREVIOUS_MINOR_RELEASE=$(git -C ${GIT_DIR} tag -l "$PREVIOUS_MINOR_RELEASE_TAG")
    if [[ -z $FOUND_PREVIOUS_MINOR_RELEASE ]]; then
      echo "Could not find previous minor release. Cannot compile diff. Aborting..."
      exit 1
    fi
  fi
  echo "Version indicates a bugfix. Only changes since $PREVIOUS_MINOR_RELEASE_TAG are packaged. "
  diff_without_add_and_modify=$(git -C ${GIT_DIR} diff --diff-filter=am --name-only "$PREVIOUS_MINOR_RELEASE_TAG..$APP_RELEASE_TAG")
  if [[ "$diff_without_add_and_modify" =~ ^[[:blank:]]+$ ]]; then
    echo "Seems that there are other chances than add/modify in sources. Bugfix allows only add/modify. Aborting..."
    exit 1
  fi
  diff_only_add_and_modify=($(git -C ${GIT_DIR} diff --diff-filter=AM --name-only "$PREVIOUS_MINOR_RELEASE_TAG..$APP_RELEASE_TAG"))
  diff_only_add_and_modify+=('upgrade.sh')
  echo "Changed files are:"
  printf " -> %s\n" "${diff_only_add_and_modify[@]}"
  git -C ${GIT_DIR} archive --format=tar ${APP_RELEASE_TAG} --prefix=src/ | tar -xf -
  printf "src/%s\n" "${diff_only_add_and_modify[@]}" | tar -cf ./vsb-${APP_RELEASE_VERSION}.tar -T -
else
  git -C ${GIT_DIR} archive --format=tar ${APP_RELEASE_TAG} --prefix=src/ > ./vsb-${APP_RELEASE_VERSION}.tar
fi
mkdir -p ./src/
echo "$APP_RELEASE_VERSION" > ./src/version;
tar -rf "vsb-$APP_RELEASE_VERSION.tar" src/version;
echo "Zipping... "
if ! gzip -f "vsb-$APP_RELEASE_VERSION.tar"; then
  echo "Could not zip sources. Aborting..."
  exit 1
fi
rm -R ./src
if [[ ${PUBLISH} == 1 ]]; then
  echo "Publishing to $RELEASE_LOCATION... "
  if [[ ${INTERACTIVE} == 1 ]]; then
    echo -n "Proceed? (y/n)"
    read answer
    if ! echo "$answer" | grep -iq "^y" ; then
      exit 0
    fi
  fi
  if ! scp "vsb-$VSB_RELEASE_VERSION.tar.gz" $RELEASE_LOCATION; then
    echo "Could not copy release to $RELEASE_LOCATION. Aborting..."
    exit 1
  fi
  echo "Dont forget wiki entry for release"
else
  echo "Publishing to $RELEASE_LOCATION was skipped. Use --publish "
fi
exit 0