#!/bin/bash
#-----------------------------------------------------------------------------------------------------------------------
#
# Upgrade Vschoolboard Application Server
#
# USAGE: upgrade.sh
#   Options:
#
#   --non-interactive=1|0
#     Interactive mode
#     Defaults to 0
#   --env=<string>
#     can be production or development
#     Defaults to 'production'
#   --reset-authorization-grants
#     if this flag is set, all existing roles in auth module will be set to their default grants
#     in rare cases, this may be necessary
#
#-----------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------
# Check caller is root
#-----------------------------------------------------------------------------------------------------------------------
if [ "$(id -u)" != "0" ]; then
   echo "This script must be run as root" 1>&2
   exit 1
fi

#-----------------------------------------------------------------------------------------------------------------------
# Check necessary commands
#-----------------------------------------------------------------------------------------------------------------------
command -v rsync >/dev/null 2>&1 || { echo "I require rsync but it's not installed. Aborting." >&2; exit 1; }

#-----------------------------------------------------------------------------------------------------------------------
# Define which and how application folder should be synced
#-----------------------------------------------------------------------------------------------------------------------
# due to the lack of associative structures, the convention for the following lists is:
# Every EVEN entry is an application folder that should be processed by rsync.
# Every following UNEVEN entry are rsync options for this folder.
# If that is not honored, there will be chaos and disarray.
folders_upgrade_sync_opts=(
"/"         "-lptgoDr --delete --exclude=/*/"
"/app/"     "-a --delete"
"/bin/"     "-a --delete"
"/db/"      "-a --delete"
"/lib/"     "-a --delete"
"/config/"  "-a"
"/public/"  "-a --delete --exclude=/uploads --exclude=/assets"
"/vendor/"  "-a --delete --exclude=/bundle"
)
folders_bugfix_sync_opts=(
"/"         "-lptgoDr --exclude=/*/"
"/app/"     "-a"
"/bin/"     "-a"
"/db/"      "-a"
"/lib/"     "-a"
"/config/"  "-a"
"/public/"  "-a --exclude=/uploads --exculde=/assets"
"/vendor/"  "-a --exclude=/bundle"
)


#-----------------------------------------------------------------------------------------------------------------------
# Script variables
#-----------------------------------------------------------------------------------------------------------------------
APP_INSTANZ="Vschoolboard Application Server"           # Application name
SCRIPT=$(readlink -f "$0")                              # scriptname
SCRIPTPATH=$(dirname "$SCRIPT")                         # scriptfolder
APP_VERSION_FILE="$SCRIPTPATH/version"                  # new version file
APP_PARENT_DIR=$(realpath "$SCRIPTPATH/..")             # parent directory of script
APP_INSTALL_PATH="$APP_PARENT_DIR/vsb"                  # current schoolboard installation path
if [[ ! -d ${APP_INSTALL_PATH} ]]; then
  echo "Cannot find schoolboard installation at $APP_INSTALL_PATH."
  echo "For installation, read README first and run install.sh."
  exit 1
fi
APP_DATA_PATH="$APP_PARENT_DIR/data"                    # user data directory
APP_TMP_PATH="$APP_INSTALL_PATH/tmp"                    # current schoolboard temp dir
APP_PIDFILE="$APP_TMP_PATH/pids/puma.pid"               # application server pid
APP_INSTALLED_VERSION_FILE="$APP_INSTALL_PATH/version"  # installed version file
APP_GROUP=$(stat -c "%G" ${APP_INSTALL_PATH})           # application group
APP_USER=$(stat -c "%U" ${APP_INSTALL_PATH})            # application user name

#-----------------------------------------------------------------------------------------------------------------------
# Script parameters
#-----------------------------------------------------------------------------------------------------------------------
APP_ENVIRONMENT="production"
INTERACTIVE=1
APP_RESET_AUTH_GRANTS=0
while [ $# -gt 0 ]; do
  case "$1" in
    --non-interactive*)
      INTERACTIVE=0
      ;;
    --env=*)
      APP_ENVIRONMENT="${1#*=}"
      ;;
    --reset-authorization-grants*)
      APP_RESET_AUTH_GRANTS=1
      ;;
    *)
      cat <<EOF
************************************************************************************************************************
USAGE: upgrade.sh <options>

   Options:

   --non-interactive=1|0
     Interactive mode
     Defaults to 0
   --env=<string>
     can be production or development
     Defaults to 'production'
   --reset-authorization-grants
     if this flag is set, all existing roles in auth module will be set to their default grants
     in rare cases, this may be necessary

************************************************************************************************************************
EOF
      exit 1
  esac
  shift
done




#-----------------------------------------------------------------------------------------------------------------------
# check installed version and version of sources to be installed
# check wether is bugfix or full upgrade
#-----------------------------------------------------------------------------------------------------------------------
if [ -f "$APP_INSTALLED_VERSION_FILE" ] ; then
  if [ -f "$APP_VERSION_FILE" ] ; then
     APP_INSTALLED_VERSION=$(cat "$APP_INSTALLED_VERSION_FILE")
     APP_VERSION=$(cat "$APP_VERSION_FILE")
     if [[ $(cut -d'.' -f3 <<<"$APP_VERSION") > 0 ]]; then
        IS_BUGFIX=1
     else
        IS_BUGFIX=0
     fi
     LOWER_APP_VERSION=$(echo -e "$APP_INSTALLED_VERSION\n$APP_VERSION" | sort -t '.' -k 1,1 -k 2,2 -k 3,3 -k 4,4 -g | sed -n 1p)
     # the lower version is first, so this checks if a higher version is to be installed.
     if [[ "$LOWER_APP_VERSION" != "$APP_INSTALLED_VERSION" || "$APP_INSTALLED_VERSION" == "$APP_VERSION"  ]] ; then
       echo  "Cannot update from $APP_INSTALLED_VERSION to $APP_VERSION, only updates to a newer version are possible"
       exit 1
     fi
     if [[ ${IS_BUGFIX} == 1 ]]; then
       if [[ $(cut -d'.' -f1 <<<"$APP_VERSION") != $(cut -d'.' -f1 <<<"$APP_INSTALLED_VERSION") || $(cut -d'.' -f2 <<<"$APP_VERSION") != $(cut -d'.' -f2 <<<"$APP_INSTALLED_VERSION")  ]] ; then
         echo  "Cannot apply bugfix for $APP_VERSION to $APP_INSTALLED_VERSION, as minor or major version differ. Upgrade to highest version first and apply bugfix afterwards. "
         exit 1
       fi
      fi
  else
     echo "Cannot find version file $APP_SOURCE_VERSION_FILE. The sources are probably not a vschoolboard application. The file should contain the version to be installed."
     exit 1
  fi
else
  echo  "Cannot find version file $APP_VERSION_FILE. This is probably not a vschoolboard application. The file should contain the installed version."
  exit 1
fi


#-----------------------------------------------------------------------------------------------------------------------
# print configuration and ask user back
#-----------------------------------------------------------------------------------------------------------------------
echo "--------------------------------------------------------"
echo "Begin update:                               $APP_INSTANZ"
echo "Updated environment is:                     $APP_ENVIRONMENT"
echo "Update is a bugfix:                         $IS_BUGFIX"
echo "Installed Version:                          $APP_INSTALLED_VERSION"
echo "New Version:                                $APP_VERSION"
echo "Upgrade will be done as user:group:         $APP_USER:$APP_GROUP"
echo "--------------------------------------------------------"
if [[ ${INTERACTIVE} == 1 ]]; then
  echo -n "Proceed? (y/n)"
  read answer
  if ! echo "$answer" | grep -iq "^y" ; then
    exit 0
  fi
  echo "Stopping application server..."
  if ! ${APP_INSTALL_PATH}/srvctl.sh stop ${APP_ENVIRONMENT}; then
    exit 1
  fi
  echo "Stopped."
fi

# working in source folder
cd ${SCRIPTPATH}

#-----------------------------------------------------------------------------------------------------------------------
# make application user and group own the sources folder
#-----------------------------------------------------------------------------------------------------------------------
echo "Changing ownership of sources to $APP_USER:$APP_GROUP"
if ! chown -R "$APP_USER:$APP_GROUP" ${SCRIPTPATH}; then
  echo "Could not change ownership of sources. Abort."
  exit 1
fi
if ! chmod -R 750 ${SCRIPTPATH}; then
  echo "Could not change access rights of sources. Abort."
  exit 1
fi

#-----------------------------------------------------------------------------------------------------------------------
# backup current installation directory
#-----------------------------------------------------------------------------------------------------------------------
APP_BACKUP=${APP_PARENT_DIR}/vsb_bck
echo "Backup existing application to $APP_BACKUP"
# backup application
if [[ ! -d $APP_BACKUP ]]; then
  if ! cp -a ${APP_INSTALL_PATH} ${APP_BACKUP} ; then
    echo "Could not backup current installation. Abort."
    exit 1
  fi
fi

# adjust access rights after backup
chown -R $APP_USER:$APP_GROUP "$APP_INSTALL_PATH"
chmod -R 750 "$APP_INSTALL_PATH"

#-----------------------------------------------------------------------------------------------------------------------
# in case of error, restore the backup
#-----------------------------------------------------------------------------------------------------------------------
function restore_backup() {
  echo $1 # reason why rollback
  if ! sudo -u ${APP_USER} rm -r ${APP_INSTALL_PATH}; then
    echo "Could not rollback. Backup folder must be moved to original place manually."
    echo "Backup: $APP_BACKUP"
    echo "Origin: $APP_INSTALL_PATH"
    exit 1
  fi
  if ! sudo -u ${APP_USER} mv ${APP_BACKUP} ${APP_INSTALL_PATH}; then
    echo "Could not rollback. Backup folder must be moved to original place manually."
    echo "Backup: $APP_BACKUP"
    echo "Origin: $APP_INSTALL_PATH"
    exit 1
  fi
  echo "Changes rolled back."
  exit 1
}


#-----------------------------------------------------------------------------------------------------------------------
# rsync a folder below APP_INSTALL_PATH
# sync_folder name <rsync options>
#-----------------------------------------------------------------------------------------------------------------------
function sync_folder {
  local folder_src="$1"
  local folder_dest="$1"
  local rsync_opts="$2"
  echo "Sync $1"
#  echo "rsync ${rsync_opts} ${SCRIPTPATH}${folder_src} ${APP_INSTALL_PATH}${folder_dest}"
#  echo -n "Proceed?"
#  read answer
  if [[ -d "${SCRIPTPATH}${folder_src}" ]]; then
    if ! sudo -u ${APP_USER} rsync ${rsync_opts} ${SCRIPTPATH}${folder_src} ${APP_INSTALL_PATH}${folder_dest}; then
      restore_backup "Could not sync $folder_src"
    fi
  fi
}


#-----------------------------------------------------------------------------------------------------------------------
# if it is a bugfix, everything is synced without delete option
#-----------------------------------------------------------------------------------------------------------------------
if [[ $IS_BUGFIX == "1" ]]; then
  folders_sync=("${folders_bugfix_sync_opts[@]}")
else
  folders_sync=("${folders_upgrade_sync_opts[@]}")
fi

#-----------------------------------------------------------------------------------------------------------------------
# sync application sources as defined in folders_sync array
#-----------------------------------------------------------------------------------------------------------------------
counter=0
while [  $counter -lt ${#folders_sync[@]} ]; do
  # sync_folder <foldername> <rsync options>
  sync_folder "${folders_sync[$counter]}" "${folders_sync[$counter+1]}"
  let counter=counter+2
done

echo "Sources synced!"
echo "Enter application root.."
cd $APP_INSTALL_PATH
#-----------------------------------------------------------------------------------------------------------------------
# Application source files are completed here. Any rake tasks can be executed after this point.
# exec_rake_task assets:clean
#-----------------------------------------------------------------------------------------------------------------------
function bundler_execute {
  if ! sudo -u ${APP_USER} RAILS_ENV=${APP_ENVIRONMENT} $( command -v bundle ) $1; then
    restore_backup "$1 failed."
  fi
}


echo "Update gems..."
bundler_execute "install --deployment";
chown -R $APP_USER:$APP_GROUP "$APP_INSTALL_PATH/vendor/bundle"
chmod -R 775 "$APP_INSTALL_PATH/vendor/bundle"
echo "Gems updated..."
#if ! sudo -u ${APP_USER} RAILS_ENV=${APP_ENVIRONMENT} $( command -v bundle ) exec rake assets:check > /dev/null 2>&1; then
  echo "Clean and recompile assets..."
  bundler_execute "exec rake assets:clean" ;
  bundler_execute "exec rake assets:precompile" 2> /dev/null;
  echo "Assets ready..."
#fi
echo "Authorization module update..."
bundler_execute "exec rake vsb:sync_authorization_items";
bundler_execute "exec rake vsb:reset_auth_role_permissions";
echo "Authorization module ready..."


#-----------------------------------------------------------------------------------------------------------------------
# Schema migrations run last, because of rollback
#-----------------------------------------------------------------------------------------------------------------------
echo "Running db schema migrations... "
bundler_execute "exec rake db:migrate";
echo "Schema db schema migrations successful."

if [[ ${INTERACTIVE} == 1 ]]; then
  echo -n "Restart application server? (y/n)"
  read answer
  if echo "$answer" | grep -iq "^y" ; then
    echo "Restart..."
    sudo -u ${APP_USER} ${APP_INSTALL_PATH}/srvctl.sh start ${APP_ENVIRONMENT}
    echo "Restarted."
  fi
fi

if [[ ${INTERACTIVE} == 1 ]]; then
  echo -n "Delete application backup? (y/n)"
  read answer
  if echo "$answer" | grep -iq "^y" ; then
    echo "Removing backup..."
    rm -R "$APP_BACKUP"
    echo "Backup removed."
  fi
fi

if [[ ${APP_RESET_AUTH_GRANTS} == 1 ]]; then
  sudo -u ${APP_USER} RAILS_ENV=${APP_ENVIRONMENT} $( command -v bundle ) exec rake vsb:reset_auth_role_permissions
fi

#-----------------------------------------------------------------------------------------------------------------------
# Restart application server
#-----------------------------------------------------------------------------------------------------------------------
echo "Update to Version $APP_VERSION complete."
exit 0
